from random import random
import time
import logging
from . import Monitor
import os


class LOGMonitor(Monitor):
    created = False

    def __init__(self, name: str):
        super().__init__(name)

    def create(self):
        logging.basicConfig(
            format=f"[{os.getpid()}]->%(asctime)s %(levelname)-4s:%(message)s",
            filename=f"{self.name}.log",
            encoding="utf-8",
            level=logging.INFO,
            datefmt="%Y-%m-%d %H:%M:%S",
            pid=os.getpid(),
        )
        self.created = True

    def track_put(self):
        if not self.created:
            self.create()
        self.put_counter += 1

        logging.info(f"put counter: {self.put_counter}")

    def track_get(self):
        if not self.created:
            self.create()
        self.get_counter += 1
        logging.info(f"get counter: {self.get_counter}")



from random import random
import time

from opencensus.ext.stackdriver import stats_exporter
from opencensus.stats import aggregation
from opencensus.stats import measure
from opencensus.stats import stats
from opencensus.stats import view

from . import Monitor


class GCPMonitor(Monitor):
    created = False

    def __init__(self, name: str):
        super().__init__(name)

    def create(self):
        self.created = True

        self.m_put = measure.MeasureInt(f"mnqueues.put", "number of puts", "1")
        self.m_get = measure.MeasureInt("mnqueues.get", "number of gets", "1")
        self.v_put = view.View(
            "mnqueues.number_queue_put",
            "number of put() to queues",
            [],
            self.m_put,
            aggregation.CountAggregation(),
        )
        self.v_get = view.View(
            "mnqueues.number_queue_get",
            "number of get() to queues",
            [],
            self.m_get,
            aggregation.CountAggregation(),
        )
        stats.stats.view_manager.register_view(self.v_put)
        stats.stats.view_manager.register_view(self.v_get)
        exporter = stats_exporter.new_stats_exporter()
        stats.stats.view_manager.register_exporter(exporter)

    def track_put(self):
        if not self.created:
            self.create()
        self.put_counter += 1
        mmap = stats.stats.stats_recorder.new_measurement_map()
        mmap.measure_int_put(self.m_put, self.put_counter)
        mmap.record()

    def track_get(self):
        if not self.created:
            self.create()
        self.get_counter += 1
        mmap = stats.stats.stats_recorder.new_measurement_map()
        mmap.measure_int_put(self.m_get, self.get_counter)
        mmap.record()



from multiprocessing import Queue
from typing import Optional

__version__ = "0.0.13"


class Monitor:
    def __init__(self, name: str):
        self.name = name
        self.put_counter: int = 0
        self.get_counter: int = 0

    def track_put(self):
        pass

    def track_get(self):
        pass


class MNQueue:
    def __init__(self, monitor: Optional[Monitor] = None, maxsize=0):
        self.monitor = monitor
        self.queue = Queue(maxsize)

    def put(self, *args, **kwargs):
        if self.monitor:
            try:
                self.monitor.track_put()
            except Exception as e:
                print(f"failed to track put() with {e}")

        return self.queue.put(*args, **kwargs)

    def get(self, *args, **kwargs):
        if self.monitor:
            try:
                self.monitor.track_get()
            except Exception as e:
                print(f"failed to track get() with {e}")

        return self.queue.get(*args, **kwargs)

    def qsize(self):
        return self.queue.qsize()

    def empty(self):
        return self.queue.empty()

    def full(self):
        return self.queue.full()

    def get_nowait(self):
        return self.get(False)

    def put_nowait(self, obj):
        return self.put(obj, False)

    def close(self):
        self.queue.close()

    def join_thread(self):
        self.queue.join_thread()

    def cancel_join_thread(self):
        self.queue.cancel_join_thread()
